/******************************************************************************
 *
 * Copyright:
 *    (C) 2006 Sulaiman Vali
 *
 * File:
 *    fifteen.c
 *
 * Description:
 *    Implementation of fifteen game.
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "../pre_emptive_os/api/osapi.h"
#include "../pre_emptive_os/api/general.h"
#include <printf_P.h>
#include <ea_init.h>
#include <stdlib.h>
#include "lcd.h"
#include "key.h"
#include "select.h"
#include "tiles.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/
#define GAME_NOT_STARTED 0
#define GAME_RUNNING     1
#define GAME_OVER        2
#define GAME_END         3

#define GRID_COLS 4
#define GRID_ROWS 4
#define OFFSET 7
#define TILEW 29
#define TILEH 29
#define MAX_TILES (GRID_COLS*GRID_ROWS)

/*****************************************************************************
 * Local variables
 ****************************************************************************/
static tU8 gameStatus;
static tU16 moves = 0;
static tS8 colIndex;
static tS8 rowIndex;
static char movesStr[] = " Moves:     ";

static tU8 board[GRID_ROWS][GRID_COLS];
static unsigned char *tile_img[] = {
  _tile1,_tile2,_tile3,_tile4,
  _tile5,_tile6,_tile7,_tile8,
  _tile9,_tile10,_tile11,_tile12,
  _tile13,_tile14,_tile15,_tile16
};

/*****************************************************************************
 * External variables
 ****************************************************************************/
extern volatile tU32 ms;

/*****************************************************************************
 *
 * Description:
 *    Check whether the move is legal/possible.
 *
 ****************************************************************************/
static tBool
isValid (tU8 key)
{
  switch(key) {
    case KEY_UP:
         if (rowIndex >= GRID_ROWS-1)
            return FALSE;
         break;
    case KEY_RIGHT:
         if (colIndex == 0)
            return FALSE;
         break;
    case KEY_DOWN:
         if (rowIndex == 0)
            return FALSE;
         break;
    case KEY_LEFT:
         if (colIndex >= GRID_COLS-1)
            return FALSE;
  }
  return TRUE;
}

/*****************************************************************************
 *
 * Description:
 *    Draw the score or whatever represents it.
 *
 ****************************************************************************/
static void
drawMoves()
{
   movesStr[7]  = moves<1000 ? ' ' : moves/1000 + '0';
   movesStr[8]  = moves<100 ? ' ' : (moves%1000)/100 + '0';
   movesStr[9]  = moves<10 ? ' ' : (moves%100)/10 + '0';
   movesStr[10] = moves%10 + '0';

   lcdColor(1, 0xff);
   lcdGotoxy(16, 116);
   lcdPuts(movesStr);
}

/*****************************************************************************
 *
 * Description:
 *    Given a particular key press, move the tile appropriately if possible.
 *
 ****************************************************************************/
static void
moveTile(tU8 key)
{
  unsigned char *img;
  tU8 destRow = rowIndex;
  tU8 destCol = colIndex;
  tU8 i, j;

  if (isValid(key))
  {
    switch(key) {
      case KEY_UP:
           rowIndex++;
           break;
      case KEY_RIGHT:
           colIndex--;
           break;
      case KEY_DOWN:
           rowIndex--;
           break;
      case KEY_LEFT:
           colIndex++;
    }

    board[destRow][destCol] = board[rowIndex][colIndex];
    board[rowIndex][colIndex] = MAX_TILES;
    lcdRect(OFFSET+colIndex*TILEW, rowIndex*TILEH, TILEW, TILEH, 0xff);
    img = tile_img[board[destRow][destCol]-1];
    lcdIcon(OFFSET+destCol*TILEW, destRow*TILEH, TILEW, TILEH, img[2], img[3], &img[4]);
    moves++;
    drawMoves();

#ifdef DEBUG
    printf("sR:%d sC:%d\t\tdR:%d dC:%d Tile:%d", rowIndex, colIndex, destRow, destCol,board[destRow][destCol]);
    for (i=0; i<GRID_ROWS; i++)
    {
      for (j=0; j<GRID_COLS; j++)
        printf("%d  ", board[i][j]);
      printf("\n");
    }
    printf("\n");
#endif
  }
}

/*****************************************************************************
 *
 * Description:
 *    Draw game background and game board, initialize all variables
 *
 ****************************************************************************/
static void
setupGame (tBool drawBoard)
{
  tU8 i, j, x;

  //initialize random generator and reset moves
  srand(ms);
  moves = 0;

  //draw background
  lcdRect(0, 0, 130, 130, 0);

  //draw game board
  if (drawBoard == TRUE)
     drawMoves();

  //initialise game board
  for (i=0; i<GRID_ROWS; i++)
    for (j=0; j<GRID_COLS; j++)
    {
      x = (i*GRID_COLS)+j;
      board[i][j] = x+1;
      if(x==MAX_TILES-1 && drawBoard)
        lcdRect(OFFSET+(TILEW*j), TILEH*i, TILEW, TILEH, 0xff);
      else
        lcdIcon(OFFSET+(TILEW*j), TILEH*i, TILEW, TILEH, tile_img[x][2], tile_img[x][3], &(tile_img[x])[4]);
    }

  //currFigure = rand() % ;
  colIndex  = GRID_COLS-1;
  rowIndex  = GRID_ROWS-1;
}


/*****************************************************************************
 *
 * Description:
 *    Implements example game
 *
 ****************************************************************************/
void
playFifteen(void)
{
  gameStatus = GAME_NOT_STARTED;

  setupGame(FALSE);
  lcdGotoxy(5,40);
  lcdColor(0,0xe0);
  lcdPuts("Press to start");

  while(gameStatus != GAME_END)
  {
    tU8 anyKey;

    anyKey = checkKey();
    switch(gameStatus)
    {
      case GAME_NOT_STARTED:
        if (anyKey != KEY_NOTHING)
        {
          gameStatus = GAME_RUNNING;
          setupGame(TRUE);
        }
        break;

      case GAME_RUNNING:
        if (anyKey != KEY_NOTHING)
        {
            if (anyKey == KEY_CENTER)
               gameStatus = GAME_OVER;
            else
               moveTile(anyKey);
        }
        break;

      case GAME_OVER:
      {
        tMenu menu;

        menu.xPos = 10;
        menu.yPos = 40;
        menu.xLen = 6+(12*8);
        menu.yLen = 4*14;
        menu.noOfChoices = 2;
        menu.initialChoice = 0;
        menu.pHeaderText = "Game over!";
        menu.headerTextXpos = 20;
        menu.pChoice[0] = "Restart game";
        menu.pChoice[1] = "End game";
        menu.bgColor       = 0;
        menu.borderColor   = 0x6d;
        menu.headerColor   = 0;
        menu.choicesColor  = 0xfd;
        menu.selectedColor = 0xe0;

        switch(drawMenu(menu))
        {
          case 0: gameStatus = GAME_RUNNING; setupGame(TRUE); break;  //Restart game
          case 1: gameStatus = GAME_END; break;                       //End game
          default: break;
        }
      }
        break;

      default:
        gameStatus = GAME_END;
        break;
    }
  }
}

